/*
 * DDUMP - Convert a dump image done on a Ti-89/Ti-92+/V200 system to a text file.
 * Copyright (C) 2003 PpHd
 *
 * This program is free software ; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along with this program;
 * if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

typedef enum {END = 0, VECTOR, EVHK_HOOK, UNK_HOOK, SYM, HANDLES, HEAP, PARM_BLOCK, AMS, PATCH, ARCHIVE, APPS, DUMP, OTH_EXT, KERNEL_EXT, PACKARC_EXT, PACKARC_FILE  } 
	DUMP_TYPE;

unsigned long ReadLong(unsigned char *c)
{
	return (c[0]*16777216+c[1]*65536+c[2]*256+c[3]);
}

unsigned short ReadShort(unsigned char *c)
{
	return (c[0]*256+c[1]);
}

void	main(int argc, char *argv[])
{
	FILE *F;
	unsigned char *file, *data;
	unsigned long len, num;
	DUMP_TYPE	type = 1;
	char	Buffer[20];
	unsigned long kernel = 0;
	
	// Read file
	if (argc < 3)
		{
		printf("Error : DDump InputFile OutputFile");
		exit(1);
		}
	file = malloc(70000);
	F = fopen(argv[1],"rb");
	if (F == NULL)
		{
		printf("Error: cannot input file");
		exit(2);
		}
	fread(file, 70000, 1, F);
	fclose(F);
	
	// Open output
	F = fopen(argv[2],"w");
	if (F == NULL)
		{
		printf("Error: cannot output file");
		exit(3);
		}
	
	// Parse Input
	data = file + 0x58;
	
	while (type != END)
		{
		type = *(data++);
		len = *(data++) *2;
		switch (type)
			{
			case AMS:
				fprintf(F,"Product Name: %s\nProduct Id: %s\n",
					data, data+40);
				switch (*(data+41))
					{
					case '1':
						fprintf(F, "TI92+ ");
						break;
					case '3':
						fprintf(F, "TI89 ");
						break;
					case '8':
						fprintf(F, "V200 ");
						break;
					default:
						fprintf(F, "Unknown calculator");
						break;
					}
				switch (*(data+45))
					{
					case '0':
						fprintf(F,"AMS 1.00\n");
						break;
					case '1':
						fprintf(F,"AMS 1.01\n");
						break;
					case '2':
						fprintf(F,"AMS 1.05\n");
						break;
					case '3':
						fprintf(F,"AMS 2.00\n");
						break;
					case '4':
						fprintf(F,"AMS 2.02\n");
						break;
					case '5':
						fprintf(F,"AMS 2.01\n");
						break;
					case '6':
						fprintf(F,"AMS 2.02\n");
						break;
					case '7':
						fprintf(F,"AMS 2.03\n");
						break;
					case '8':
						fprintf(F,"AMS 2.04\n");
						break;
					case '9':
						fprintf(F,"AMS 2.05\n");
						break;
					case 'A':
						fprintf(F,"AMS 2.07\n");
						break;
					case 'B':
						fprintf(F,"AMS 2.08\n");
						break;
					default:
						fprintf(F,"Unknown AMS\n");
						break;
					}
				break;
			case VECTOR:
				num = ReadShort(data);
				fprintf(F,"Vector[$%02X]: Addr=%08X Dump:%02X %02X %02X %02X %02X %02X %02X %02X\n",
					num*4, ReadLong(data+2),
					data[6], data[7], data[8], data[9], data[10], data[11], data[12], data[13]);
				if (num == 0x0C)
					kernel = ReadLong(data+2);
				break;
			case HEAP:
				fprintf(F,"HEAP - Free Handles: %d\n     Mem Available: %ld \n     Max Handle Size: %ld\n\n",
					ReadShort(data),
					ReadLong(data+2),
					ReadLong(data+6));
				break;
			case PARM_BLOCK:
				fprintf(F, "HARDWARE PARM BLOCK : Len: %d\n"
					"    Hardware Id : %ld\n"
					"    Hardware Revision: %ld\n"
					"    Boot Major : %ld\n"
					"    Boot Revision : %ld\n"
					"    Boot Build : %ld\n"
					"    Gate Array : %ld\n"
					"    PhysDisplayBitsWide: %ld\n"
					"    PhysDisplayBitsTall: %ld\n"
					"    LCDBitsWide: %ld\n"
					"    LCDBitsTall: %ld\n\n",
					len,
					ReadLong(data+0),
					ReadLong(data+4),
					ReadLong(data+8),
					ReadLong(data+12),
					ReadLong(data+16),
					ReadLong(data+20),
					ReadLong(data+24),
					ReadLong(data+28),
					ReadLong(data+32),
					ReadLong(data+36));
				break;
			case HANDLES:
				fprintf(F,"Handle[%03X]: Addr=%08X Size:%04X Dump:%02X %02X %02X %02X %02X %02X %02X %02X\n",
					ReadShort(data), ReadLong(data+2), ReadShort(data+6),
					data[8], data[9], data[10], data[11], data[12], data[13], data[14], data[15]);
				break;
			case PATCH:
				fprintf(F, "\nDetected Hardware Version: %ld\n",
					ReadShort(data));
				if (ReadShort(data+2))
					fprintf(F,"HW2PATCH Detected\n");
				else	fprintf(F,"HW2PATCH Not Detected\n");
				if (ReadShort(data+4))
					fprintf(F,"HW220XTSR Detected\n");
				else	fprintf(F,"HW220XTSR Not Detected\n");
				break;
			case ARCHIVE:
				fprintf(F, "\nArchive InUSe: %ld\n"
					"        Free AfterGC: %ld\n"
					"        Free: %ld\n"
					"        Unused Sectors: %ld\n"
					"        Bad Sectors: %ld\n"
					"        Execpt Base Code: %ld\n",
					ReadLong(data),
					ReadLong(data+4),
					ReadLong(data+8),
					ReadLong(data+12),
					ReadLong(data+16),
					ReadLong(data+20));
				break;
			case DUMP:
				fprintf(F, "DUMP Program Version: %04X\n",
					ReadShort(data));
				break;
			case EVHK_HOOK:
				strncpy(Buffer, data+4, 8);
				Buffer[8]= 0;
				fprintf(F,"KK EV_Hook: Addr=%08X Name:%s Next EV_Hook: %08X \n"
					"    Dump:%02X %02X %02X %02X %02X %02X %02X %02X\n",
					ReadLong(data), Buffer, ReadLong(data+12),
					data[16], data[17], data[18], data[19], data[20], data[21], data[22], data[23]);
				break;
			case UNK_HOOK:
				fprintf(F,"Unknown EV_Hook: Addr=%08X Dump:%02X %02X %02X %02X %02X %02X %02X %02X\n",
					ReadLong(data),
					data[4], data[5], data[6], data[7], data[8], data[9], data[10], data[11]);
				break;
			case APPS:
				fprintf(F, "Application : %s\n", data);
				break;
			case SYM:
				fprintf(F, "SYM[%8s\\%-8s] = Compat: %04X Flags: %04X Handle: %03X Type:%02X CheckSum: %d\n",
					data, data+10, ReadShort(data+18), ReadShort(data+20), ReadShort(data+22), data[25], ReadShort(data+26));
				break; 
			case OTH_EXT:
				fprintf(F, " OTH extension: %s\n", data);
				break; 
			case KERNEL_EXT:
				fprintf(F, " KERNEL: Version= %02X Flags= %02X Comment= %s\n",
					data[0], data[1], data+2);
				break; 
			case PACKARC_EXT:
				fprintf(F, " PACK ARCHIVE: NumberOfFiles= %d LibraryMinVersion= %d LibraryFunction= %d LibraryName= %s\n",
					data[0], data[1], data[2], data+3);
				break; 
			case PACKARC_FILE:
				if (data[0] != '!')
					fprintf(F, "     CompressedSubFile: %s\n", data);
				else	fprintf(F, "     UnCompressedSubFile: %s\n", data+3);
				break; 
			default:
				break;
			}
		data += len;
		}

	// Kernel Info
	if (kernel)
		{
		char *kernel_str;
		
		if ( (kernel&0xFFFF) == 0x4B4E) //0x4B4E==#'KN'==KerNO
			kernel_str = "KerNO";
		else	if((kernel&0xFFFF) == 0x554F)//0x554F==#'UO'==Universal OS
			kernel_str = "Universal OS";
		else	if((kernel&0xFFFF) == 0x4c58)//0x4c58==#'LX'==Lex OS
			kernel_str = "Lex OS";
		else	if((kernel&0xFFFF) == 0x544F)//0x544F==#'TO'==TEOS
			kernel_str = "TEOS";
		else	if((kernel&0xFFFF) == 0x5053)//0x5053==#'PS'==Plusshell
			kernel_str = "PlusShell";
		else	if((kernel&0xFFFF) == 0x4454)//0x4454==#'DN'==Doors OS
			kernel_str = "Doors OS";
		else  if((kernel&0xFFFF) == 0x504F)//0x504F==#'PO'==PreOS
			kernel_str = "PreOS";
		else  if((kernel&0xFFFF) == 0x524F)	// 'RO' == PedroM
			kernel_str = "PedroM";
		else  if((kernel&0xFFFF) == 0x454B) // 'EK' = Enhanced Kernel
			kernel_str = "Enhanced Kernel";
		else	kernel_str = "Unknown Kernel";
		fprintf(F, "\nKernel: %s (TAG: %04X) Version: %04X\n",
			kernel_str, (kernel&0xFFFF), kernel >>16);
		}
	else	fprintf(F, "\nNo kernel detected\n");
	// End of file
	fclose(F);
	free(file);
}

